@@ -160,6 +160,39 @@ namespace NuGet
             };
         }
 
+        public static Manifest Create(PackageBuilder packageBuilder)
+        {
+            var metadata = (IPackageMetadata)packageBuilder;
+
+            return new Manifest
+            {
+                Metadata = new ManifestMetadata
+                {
+                    Id = metadata.Id.SafeTrim(),
+                    Version = metadata.Version.ToStringSafe(),
+                    Title = metadata.Title.SafeTrim(),
+                    Authors = GetCommaSeparatedString(metadata.Authors),
+                    Owners = GetCommaSeparatedString(metadata.Owners) ?? GetCommaSeparatedString(metadata.Authors),
+                    Tags = String.IsNullOrEmpty(metadata.Tags) ? null : metadata.Tags.SafeTrim(),
+                    LicenseUrl = ConvertUrlToStringSafe(metadata.LicenseUrl),
+                    ProjectUrl = ConvertUrlToStringSafe(metadata.ProjectUrl),
+                    IconUrl = ConvertUrlToStringSafe(metadata.IconUrl),
+                    RequireLicenseAcceptance = metadata.RequireLicenseAcceptance,
+                    DevelopmentDependency = metadata.DevelopmentDependency,
+                    Description = metadata.Description.SafeTrim(),
+                    Copyright = metadata.Copyright.SafeTrim(),
+                    Summary = metadata.Summary.SafeTrim(),
+                    ReleaseNotes = metadata.ReleaseNotes.SafeTrim(),
+                    Language = metadata.Language.SafeTrim(),
+                    DependencySets = CreateDependencySets(metadata),
+                    FrameworkAssemblies = CreateFrameworkAssemblies(metadata),
+                    ReferenceSets = CreateReferenceSets(metadata),
+                    MinClientVersionString = metadata.MinClientVersion.ToStringSafe(),
+                    ContentFiles = packageBuilder.ContentFiles.ToList()
+                },
+            };
+        }
+
         private static string ConvertUrlToStringSafe(Uri url)
         {
             if (url != null)
@@ -221,7 +254,9 @@ namespace NuGet
                     select new ManifestDependency
                     {
                         Id = dependency.Id.SafeTrim(),
-                        Version = dependency.VersionSpec.ToStringSafe()
+                        Version = dependency.VersionSpec.ToStringSafe(),
+                        Include = dependency.Include,
+                        Exclude = dependency.Exclude
                     }).ToList();
         }
 
