@@ -1,5 +1,4 @@
 ﻿using System;
-using System.Collections;
 using System.Collections.Concurrent;
 using System.Collections.Generic;
 using System.Diagnostics.CodeAnalysis;
@@ -20,19 +19,12 @@ namespace NuGet
         private readonly Lazy<bool> _supportsPrereleasePackages;
 
         private const string SourceValue = "(Aggregate source)";
-        private ILogger _logger;
 
         public override string Source
         {
             get { return SourceValue; }
         }
 
-        public ILogger Logger
-        {
-            get { return _logger ?? NullLogger.Instance; }
-            set { _logger = value; }
-        }
-
         /// <summary>
         /// Determines if dependency resolution is performed serially on a per-repository basis. The first repository that has a compatible dependency 
         /// regardless of version would win if this property is true.
