@@ -1,5 +1,6 @@
 ﻿using System;
 using System.Collections.Generic;
+using System.Collections.ObjectModel;
 using System.ComponentModel.DataAnnotations;
 using System.Globalization;
 using System.IO;
@@ -16,6 +17,239 @@ namespace NuGet.Test
 {
     public class PackageBuilderTest
     {
+        [Fact]
+        public void CreatePackageWithEmptyFoldersForV3Folders()
+        {
+            // Arrange
+            PackageBuilder builder = new PackageBuilder()
+            {
+                Id = "A",
+                Version = new SemanticVersion("1.0"),
+                Description = "Descriptions",
+            };
+
+            builder.Authors.Add("testAuthor");
+
+            var dependencies = new List<PackageDependency>();
+            dependencies.Add(new PackageDependency("packageB", VersionUtility.ParseVersionSpec("1.0.0"), null, "z"));
+            dependencies.Add(new PackageDependency(
+                "packageC",
+                VersionUtility.ParseVersionSpec("1.0.0"),
+                "a,b,c",
+                "b,c"));
+
+            var set = new PackageDependencySet(null, dependencies);
+            builder.DependencySets.Add(set);
+
+            builder.Files.Add(CreatePackageFile(@"build\_._"));
+            builder.Files.Add(CreatePackageFile(@"content\_._"));
+            builder.Files.Add(CreatePackageFile(@"contentFiles\any\any\_._"));
+            builder.Files.Add(CreatePackageFile(@"lib\net45\_._"));
+            builder.Files.Add(CreatePackageFile(@"native\net45\_._"));
+            builder.Files.Add(CreatePackageFile(@"ref\net45\_._"));
+            builder.Files.Add(CreatePackageFile(@"runtimes\net45\_._"));
+            builder.Files.Add(CreatePackageFile(@"tools\_._"));
+
+            using (var ms = new MemoryStream())
+            {
+                // Act
+                builder.Save(ms);
+
+                ms.Seek(0, SeekOrigin.Begin);
+
+                var zip = new ZipPackage(ms);
+                var files = zip.GetFiles()
+                    .Select(file => file.Path)
+                    .Where(file => Path.GetFileName(file) == "_._")
+                    .OrderBy(s => s)
+                    .ToArray();
+
+                // Assert
+                Assert.Equal(8, files.Length);
+                Assert.Equal(@"build\_._", files[0]);
+                Assert.Equal(@"content\_._", files[1]);
+                Assert.Equal(@"contentFiles\any\any\_._", files[2]);
+                Assert.Equal(@"lib\net45\_._", files[3]);
+                Assert.Equal(@"native\net45\_._", files[4]);
+                Assert.Equal(@"ref\net45\_._", files[5]);
+                Assert.Equal(@"runtimes\net45\_._", files[6]);
+                Assert.Equal(@"tools\_._", files[7]);
+            }
+        }
+
+        [Fact]
+        public void CreatePackageWithNuspecIncludeExcludeAnyGroup()
+        {
+            // Arrange
+            PackageBuilder builder = new PackageBuilder()
+            {
+                Id = "A",
+                Version = new SemanticVersion("1.0"),
+                Description = "Descriptions",
+            };
+            builder.Authors.Add("testAuthor");
+
+            var dependencies = new List<PackageDependency>();
+            dependencies.Add(new PackageDependency("packageB", VersionUtility.ParseVersionSpec("1.0.0"), null, "z"));
+            dependencies.Add(new PackageDependency(
+                "packageC",
+                VersionUtility.ParseVersionSpec("1.0.0"),
+                "a,b,c",
+                "b,c"));
+
+            var set = new PackageDependencySet(null, dependencies);
+            builder.DependencySets.Add(set);
+
+            using (var ms = new MemoryStream())
+            {
+                builder.Save(ms);
+
+                ms.Seek(0, SeekOrigin.Begin);
+
+                var manifestStream = GetManifestStream(ms);
+
+                var result = manifestStream.ReadToEnd();
+
+                // Assert
+                Assert.Equal(@"<?xml version=""1.0""?>
+<package xmlns=""http://schemas.microsoft.com/packaging/2013/05/nuspec.xsd"">
+  <metadata>
+    <id>A</id>
+    <version>1.0</version>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
+    <requireLicenseAcceptance>false</requireLicenseAcceptance>
+    <description>Descriptions</description>
+    <dependencies>
+      <group>
+        <dependency id=""packageB"" version=""1.0.0"" exclude=""z"" />
+        <dependency id=""packageC"" version=""1.0.0"" include=""a,b,c"" exclude=""b,c"" />
+      </group>
+    </dependencies>
+  </metadata>
+</package>".Replace("\r\n", "\n"), result.Replace("\r\n", "\n"));
+            }
+        }
+
+        [Fact]
+        public void CreatePackageWithNuspecIncludeExclude()
+        {
+            // Arrange
+            PackageBuilder builder = new PackageBuilder()
+            {
+                Id = "A",
+                Version = new SemanticVersion("1.0"),
+                Description = "Descriptions",
+            };
+            builder.Authors.Add("testAuthor");
+
+            var dependencies45 = new List<PackageDependency>();
+            dependencies45.Add(new PackageDependency("packageB", VersionUtility.ParseVersionSpec("1.0.0"), null, "z"));
+
+            var dependencies46 = new List<PackageDependency>();
+            dependencies46.Add(new PackageDependency(
+                "packageC",
+                VersionUtility.ParseVersionSpec("1.0.0"),
+                "a,b,c",
+                "b,c"));
+
+            var net45 = new PackageDependencySet(new FrameworkName(".NETFramework", new Version(4, 5)), dependencies45);
+            builder.DependencySets.Add(net45);
+
+            var net46 = new PackageDependencySet(new FrameworkName(".NETFramework", new Version(4, 6)), dependencies46);
+            builder.DependencySets.Add(net46);
+
+            using (var ms = new MemoryStream())
+            {
+                builder.Save(ms);
+
+                ms.Seek(0, SeekOrigin.Begin);
+
+                var manifestStream = GetManifestStream(ms);
+
+                var result = manifestStream.ReadToEnd();
+
+                // Assert
+                Assert.Equal(@"<?xml version=""1.0""?>
+<package xmlns=""http://schemas.microsoft.com/packaging/2013/05/nuspec.xsd"">
+  <metadata>
+    <id>A</id>
+    <version>1.0</version>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
+    <requireLicenseAcceptance>false</requireLicenseAcceptance>
+    <description>Descriptions</description>
+    <dependencies>
+      <group targetFramework="".NETFramework4.5"">
+        <dependency id=""packageB"" version=""1.0.0"" exclude=""z"" />
+      </group>
+      <group targetFramework="".NETFramework4.6"">
+        <dependency id=""packageC"" version=""1.0.0"" include=""a,b,c"" exclude=""b,c"" />
+      </group>
+    </dependencies>
+  </metadata>
+</package>".Replace("\r\n", "\n"), result.Replace("\r\n", "\n"));
+            }
+        }
+
+        [Fact]
+        public void CreatePackageWithNuspecContentV2()
+        {
+            // Arrange
+            PackageBuilder builder = new PackageBuilder()
+            {
+                Id = "A",
+                Version = new SemanticVersion("1.0"),
+                Description = "Descriptions",
+            };
+            builder.Authors.Add("testAuthor");
+            builder.Files.Add(CreatePackageFile("contentFiles\\any\\any\\config\\config.xml"));
+            builder.Files.Add(CreatePackageFile("contentFiles\\cs\\net45\\code.cs.pp"));
+
+            builder.ContentFiles.Add(new ManifestContentFiles()
+            {
+                Include = "**/*",
+                BuildAction = "Compile"
+            });
+
+            builder.ContentFiles.Add(new ManifestContentFiles()
+            {
+                Include = "**/*",
+                Exclude = "**/*.cs",
+                BuildAction = "None",
+                Flatten = "true",
+                CopyToOutput = "true"
+            });
+
+            using (var ms = new MemoryStream())
+            {
+                builder.Save(ms);
+
+                ms.Seek(0, SeekOrigin.Begin);
+
+                var manifestStream = GetManifestStream(ms);
+
+                var result = manifestStream.ReadToEnd();
+
+                // Assert
+                Assert.Equal(@"<?xml version=""1.0""?>
+<package xmlns=""http://schemas.microsoft.com/packaging/2013/05/nuspec.xsd"">
+  <metadata>
+    <id>A</id>
+    <version>1.0</version>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
+    <requireLicenseAcceptance>false</requireLicenseAcceptance>
+    <description>Descriptions</description>
+    <contentFiles>
+      <files include=""**/*"" buildAction=""Compile"" />
+      <files include=""**/*"" exclude=""**/*.cs"" buildAction=""None"" copyToOutput=""true"" flatten=""true"" />
+    </contentFiles>
+  </metadata>
+</package>".Replace("\r\n", "\n"), result.Replace("\r\n", "\n"));
+            }
+        }
+
         [Fact]
         public void OwnersFallsBackToAuthorsIfNoneSpecified()
         {
@@ -160,7 +394,7 @@ namespace NuGet.Test
                 Version = new SemanticVersion("1.0"),
                 Description = "Descriptions",
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
 
             var dependencies = new PackageDependency[] { 
                 new PackageDependency("B")
@@ -179,8 +413,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
     <dependencies>
@@ -200,7 +434,7 @@ namespace NuGet.Test
                 Version = new SemanticVersion("1.0"),
                 Description = "Descriptions",
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
 
             var fx = new FrameworkName("Silverlight", new Version("4.0"));
             var dependencies = new PackageDependency[] { 
@@ -220,8 +454,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
     <dependencies>
@@ -243,7 +477,7 @@ namespace NuGet.Test
                 Version = new SemanticVersion("1.0"),
                 Description = "Descriptions",
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
             builder.Files.Add(CreatePackageFile("content\\winrt53\\one.txt"));
 
             using (var ms = new MemoryStream())
@@ -260,8 +494,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
   </metadata>
@@ -279,7 +513,7 @@ namespace NuGet.Test
                 Version = new SemanticVersion("1.0"),
                 Description = "Descriptions",
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
             builder.Files.Add(CreatePackageFile("content\\bar\\one.txt"));
 
             using (var ms = new MemoryStream())
@@ -296,8 +530,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
   </metadata>
@@ -315,7 +549,7 @@ namespace NuGet.Test
                 Version = new SemanticVersion("1.0"),
                 Description = "Descriptions",
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
             builder.Files.Add(CreatePackageFile("tools\\sl4\\one.dll"));
 
             using (var ms = new MemoryStream())
@@ -332,8 +566,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
   </metadata>
@@ -351,7 +585,7 @@ namespace NuGet.Test
                 Version = new SemanticVersion("1.0"),
                 Description = "Descriptions",
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
             builder.Files.Add(CreatePackageFile("tools\\foo\\one.dll"));
 
             using (var ms = new MemoryStream())
@@ -368,8 +602,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
   </metadata>
@@ -390,7 +624,7 @@ namespace NuGet.Test
                 Version = new SemanticVersion("1.0"),
                 Description = "Descriptions",
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
             builder.Files.Add(CreatePackageFile(packagePath));
 
             using (var ms = new MemoryStream())
@@ -407,8 +641,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
   </metadata>
@@ -430,7 +664,7 @@ namespace NuGet.Test
                 Version = new SemanticVersion("1.0"),
                 Description = "Descriptions",
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
             builder.Files.Add(CreatePackageFile(packagePath));
 
             using (var ms = new MemoryStream())
@@ -447,8 +681,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
   </metadata>
@@ -466,7 +700,7 @@ namespace NuGet.Test
                 Version = new SemanticVersion("1.0"),
                 Description = "Descriptions",
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
             builder.Files.Add(CreatePackageFile("content\\web.config.install.xdt"));
             builder.Files.Add(CreatePackageFile("content\\app.config.uninstall.xdt"));
 
@@ -484,8 +718,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
   </metadata>
@@ -507,7 +741,7 @@ namespace NuGet.Test
                 Version = new SemanticVersion("1.0"),
                 Description = "Descriptions",
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
             builder.Files.Add(CreatePackageFile(packagePath));
 
             using (var ms = new MemoryStream())
@@ -524,8 +758,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
   </metadata>
@@ -548,7 +782,7 @@ namespace NuGet.Test
                 Version = new SemanticVersion("1.0"),
                 Description = "Descriptions",
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
             builder.Files.Add(CreatePackageFile(packagePath));
 
             using (var ms = new MemoryStream())
@@ -565,8 +799,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
   </metadata>
@@ -584,7 +818,7 @@ namespace NuGet.Test
                 Version = new SemanticVersion("1.0"),
                 Description = "Descriptions",
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
             builder.PackageAssemblyReferences.Add(
                 new PackageReferenceSet(
                     new FrameworkName(".NET, Version=3.0"),
@@ -605,8 +839,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
     <references>
@@ -630,7 +864,7 @@ namespace NuGet.Test
                 Description = "Descriptions",
                 DevelopmentDependency = true
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
             builder.PackageAssemblyReferences.Add(
                 new PackageReferenceSet(
                     new FrameworkName(".NET, Version=3.0"),
@@ -651,8 +885,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <developmentDependency>true</developmentDependency>
     <description>Descriptions</description>
@@ -676,7 +910,7 @@ namespace NuGet.Test
                 Version = new SemanticVersion("1.0"),
                 Description = "Descriptions",
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
             builder.PackageAssemblyReferences.Add(
                 new PackageReferenceSet(
                     null,
@@ -697,8 +931,8 @@ namespace NuGet.Test
   <metadata>
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
     <references>
@@ -720,7 +954,7 @@ namespace NuGet.Test
                 Description = "Descriptions",
                 MinClientVersion = new Version("2.0")
             };
-            builder.Authors.Add("JaneDoe");
+            builder.Authors.Add("testAuthor");
             builder.Files.Add(CreatePackageFile("a.txt"));
 
             using (var ms = new MemoryStream())
@@ -737,8 +971,8 @@ namespace NuGet.Test
   <metadata minClientVersion=""2.0"">
     <id>A</id>
     <version>1.0</version>
-    <authors>JaneDoe</authors>
-    <owners>JaneDoe</owners>
+    <authors>testAuthor</authors>
+    <owners>testAuthor</owners>
     <requireLicenseAcceptance>false</requireLicenseAcceptance>
     <description>Descriptions</description>
   </metadata>
@@ -1407,7 +1641,7 @@ Description is required.");
   <metadata>
     <id>SuperPackage</id>
     <version>2.5</version>
-    <authors>JaneDoe</authors>
+    <authors>testAuthor</authors>
     <description>description</description>
     <dependencies>
         <group targetFramework=""sl4"">
@@ -1440,7 +1674,7 @@ Description is required.");
   <metadata>
     <id>SuperPackage</id>
     <version>2.5</version>
-    <authors>JaneDoe</authors>
+    <authors>testAuthor</authors>
     <description>description</description>
     <dependencies>
         <group targetFramework=""sl4"">
@@ -1493,7 +1727,7 @@ Description is required.");
   <metadata>
     <id>SuperPackage</id>
     <version>2.5</version>
-    <authors>JaneDoe</authors>
+    <authors>testAuthor</authors>
     <description>description</description>
     <dependencies>
         <dependency id=""A"" />
