@@ -1,7 +1,9 @@
 using System;
 using System.Collections.Generic;
+using System.Diagnostics;
 using System.Linq;
 using System.Runtime.Versioning;
+using Moq;
 using Xunit;
 using Xunit.Extensions;
 
@@ -9,6 +11,44 @@ namespace NuGet.Test
 {
     public class VersionUtilityTest
     {
+        /// <summary>
+        /// This is not meant to be an exhaustive list. Instead it should be a simple test to
+        /// verify that NuGet 3.x code is being called.
+        /// </summary>
+        [Theory]
+        [InlineData("net45", "netstandard1.1", true)]
+        [InlineData("net45", "netstandard1.2", false)]
+        [InlineData("net463", "netstandard1.6", true)]
+        [InlineData("net463", "netstandard1.7", false)]
+        [InlineData("netstandardapp1.6", "netstandard1.6", true)]
+        [InlineData("netstandardapp1.6", "netstandard1.7", false)]
+        [InlineData("netcoreapp1.0", "netstandard1.6", true)]
+        [InlineData("netcoreapp1.0", "netstandard1.7", false)]
+        [InlineData("xamarinmac", "netstandard1.6", true)]
+        [InlineData("xamarinmac", "netstandard1.7", false)]
+        [InlineData("portable-net45+win8", "netstandard1.1", true)]
+        [InlineData("portable-net45+win8", "netstandard1.2", false)]
+        [InlineData("portable-net451+win81", "netstandard1.2", true)]
+        [InlineData("portable-net451+win81", "netstandard1.3", false)]
+        public void SupportsFrameworksFromNuGet3(string projectFolder, string packageFolder, bool expected)
+        {
+            // Arrange
+            var project = VersionUtility.ParseFrameworkName(projectFolder);
+            var package = VersionUtility.ParseFrameworkName(packageFolder);
+
+            // Act
+            var actual = VersionUtility.IsCompatible(project, package);
+
+            // Assert
+            Assert.True(
+                expected == actual,
+                string.Format(
+                    "{0} should {1}support {2}.",
+                    projectFolder,
+                    expected ? string.Empty : "not ",
+                    packageFolder));
+        }
+
         [Fact]
         public void ParseUAPFrameworkShortName()
         {
@@ -148,7 +188,7 @@ namespace NuGet.Test
             // Act
             string effectivePath;
             var framework = VersionUtility.ParseFrameworkNameFromFilePath(path, out effectivePath);
-            
+
             // Assert
             Assert.Null(framework);
             Assert.Equal(expectedPath, effectivePath);
@@ -415,9 +455,9 @@ namespace NuGet.Test
         [InlineData(new[] { "XamarinPlayStationVita", "xamarinplaystationvita", "XAMARINPLAYSTATIONVITA " }, "0.0", "Xamarin.PlayStationVita")]
         [InlineData(new[] { "Xamarin.PlayStationVita", "xamarin.playstationvita", "XAMARIN.PLAYSTATIONVITA " }, "0.0", "Xamarin.PlayStationVita")]
         [InlineData(new[] { "XamarinPSVita", "xamarinpsvita", "XAMARINPSVITA " }, "0.0", "Xamarin.PlayStationVita")]
-        [InlineData(new[] { "XamarinTVOS", "xamarintvos", "XAMARINTVOS " }, "0.0", "Xamarin.TVOS")]  
-        [InlineData(new[] { "Xamarin.TVOS", "xamarin.tvos", "XAMARIN.TVOS " }, "0.0", "Xamarin.TVOS")]  
-        [InlineData(new[] { "XamarinWatchOS", "xamarinwatchos", "XAMARINWATCHOS " }, "0.0", "Xamarin.WatchOS")]  
+        [InlineData(new[] { "XamarinTVOS", "xamarintvos", "XAMARINTVOS " }, "0.0", "Xamarin.TVOS")]
+        [InlineData(new[] { "Xamarin.TVOS", "xamarin.tvos", "XAMARIN.TVOS " }, "0.0", "Xamarin.TVOS")]
+        [InlineData(new[] { "XamarinWatchOS", "xamarinwatchos", "XAMARINWATCHOS " }, "0.0", "Xamarin.WatchOS")]
         [InlineData(new[] { "Xamarin.WatchOS", "xamarin.watchos", "XAMARIN.WATCHOS " }, "0.0", "Xamarin.WatchOS")]
         [InlineData(new[] { "Xamarin.XboxThreeSixty", "xamarin.xboxthreesixty", "XAMARIN.XBOXTHREESIXTY " }, "0.0", "Xamarin.Xbox360")]
         [InlineData(new[] { "XamarinXboxThreeSixty", "xamarinxboxthreesixty", "XAMARINXBOXTHREESIXTY " }, "0.0", "Xamarin.Xbox360")]
@@ -799,16 +839,6 @@ namespace NuGet.Test
             Assert.True(versionInfo.IsMaxInclusive);
         }
 
-        [Fact]
-        public void NormalizeVersionFillsInZerosForUnsetVersionParts()
-        {
-            // Act
-            Version version = VersionUtility.NormalizeVersion(new Version("1.5"));
-
-            // Assert
-            Assert.Equal(new Version("1.5.0.0"), version);
-        }
-
         [Fact]
         public void ParseVersionSpecRangeIntegerRanges()
         {
@@ -861,8 +891,8 @@ namespace NuGet.Test
         public void GetAllPossibleVersionsTwoDigits()
         {
             // Arrange
-            var expectedVersions = new[] { 
-                new SemanticVersion("1.1"), 
+            var expectedVersions = new[] {
+                new SemanticVersion("1.1"),
                 new SemanticVersion("1.1.0"),
                 new SemanticVersion("1.1.0.0")
             };
@@ -878,8 +908,8 @@ namespace NuGet.Test
         public void GetAllPossibleVersionsThreeDigits()
         {
             // Arrange
-            var expectedVersions = new[] { 
-                new SemanticVersion("1.0"), 
+            var expectedVersions = new[] {
+                new SemanticVersion("1.0"),
                 new SemanticVersion("1.0.0"),
                 new SemanticVersion("1.0.0.0"),
             };
@@ -895,13 +925,13 @@ namespace NuGet.Test
         public void GetAllPossibleVersionsFourDigits()
         {
             // Arrange
-            var expectedVersions = new[] { 
-                new SemanticVersion("1.0"), 
+            var expectedVersions = new[] {
+                new SemanticVersion("1.0"),
                 new SemanticVersion("1.0.0"),
                 new SemanticVersion("1.0.0.0"),
             };
             var expectedVersionStrings = new[] {
-                "1.0", 
+                "1.0",
                 "1.0.0",
                 "1.0.0.0"
             };
@@ -918,11 +948,11 @@ namespace NuGet.Test
         public void GetAllPossibleVersionsThreeDigitsWithZeroBetween()
         {
             // Arrange
-            var expectedVersions = new[] { 
-                new SemanticVersion("1.0.1"), 
+            var expectedVersions = new[] {
+                new SemanticVersion("1.0.1"),
                 new SemanticVersion("1.0.1.0")
             };
-            var expectedVersionStrings = new[] 
+            var expectedVersionStrings = new[]
             {
                 "1.0.1",
                 "1.0.1.0",
@@ -940,12 +970,12 @@ namespace NuGet.Test
         public void GetAllPossibleVersionsFourDigitsWithTrailingZeros()
         {
             // Arrange
-            var expectedVersions = new[] { 
+            var expectedVersions = new[] {
                 new SemanticVersion("1.1"),
                 new SemanticVersion("1.1.0"),
                 new SemanticVersion("1.1.0.0"),
             };
-            var expectedVersionStrings = new[] 
+            var expectedVersionStrings = new[]
             {
                 "1.1",
                 "1.1.0",
@@ -1110,6 +1140,9 @@ namespace NuGet.Test
         [InlineData("windows8")]
         [InlineData("win")]
         [InlineData("win8")]
+        [InlineData("windows45")]
+        [InlineData("windows1")]
+        [InlineData("windows10")] // Parses to Windows v1.0
         public void WindowsIdentifierCompatibleWithWindowsStoreAppProjects(string identifier)
         {
             // Arrange
@@ -1124,10 +1157,8 @@ namespace NuGet.Test
         [Theory]
         [InlineData("windows9")]
         [InlineData("win9")]
-        [InlineData("win10")]
+        [InlineData("win10.0")]
         [InlineData("windows81")]
-        [InlineData("windows45")]
-        [InlineData("windows1")]
         public void WindowsIdentifierWithUnsupportedVersionNotCompatibleWithWindowsStoreAppProjects(string identifier)
         {
             // Arrange
@@ -1336,12 +1367,13 @@ namespace NuGet.Test
         public void TestGetShortNameForPortableFramework()
         {
             // Arrange
-            NetPortableProfileTable.Profiles = BuildProfileCollection();
+            var profileCollection = BuildProfileCollection();
+            var table = new NetPortableProfileTable(profileCollection);
 
             var framework = new FrameworkName(".NETPortable, Version=4.0, Profile=Profile1");
 
             // Act-1
-            string shortName = VersionUtility.GetShortFrameworkName(framework);
+            string shortName = VersionUtility.GetShortFrameworkName(table, framework);
 
             // Assert-2
             Assert.Equal("portable-net45+sl40+wp71", shortName);
@@ -1350,7 +1382,7 @@ namespace NuGet.Test
             var framework2 = new FrameworkName(".NETPortable, Version=4.0, Profile=Profile2");
 
             // Act-2
-            string shortName2 = VersionUtility.GetShortFrameworkName(framework2);
+            string shortName2 = VersionUtility.GetShortFrameworkName(table, framework2);
 
             // Assert-2
             Assert.Equal("portable-win+sl30+wp71", shortName2);
@@ -1359,7 +1391,7 @@ namespace NuGet.Test
             var framework3 = new FrameworkName(".NETPortable, Version=4.0, Profile=Profile4");
 
             // Act-3
-            string shortName3 = VersionUtility.GetShortFrameworkName(framework3);
+            string shortName3 = VersionUtility.GetShortFrameworkName(table, framework3);
 
             // Assert-4
             Assert.Equal("portable-sl20+wp", shortName3);
@@ -1459,7 +1491,7 @@ namespace NuGet.Test
         [InlineData("Xamarin.PlayStation3, Version=v1.0", "xamarinpsthree10")]
         [InlineData("Xamarin.PlayStation4, Version=v1.0", "xamarinpsfour10")]
         [InlineData("Xamarin.PlayStationVita, Version=v1.0", "xamarinpsvita10")]
-        [InlineData("Xamarin.TVOS, Version=v1.0", "xamarintvos10")]  
+        [InlineData("Xamarin.TVOS, Version=v1.0", "xamarintvos10")]
         [InlineData("Xamarin.WatchOS, Version=v1.0", "xamarinwatchos10")]
         [InlineData("Xamarin.Xbox360, Version=v1.0", "xamarinxboxthreesixty10")]
         [InlineData("Xamarin.XboxOne, Version=v1.0", "xamarinxboxone10")]
@@ -1476,7 +1508,7 @@ namespace NuGet.Test
         [InlineData(".NETPortable, Version=4.0, Profile=Profile1", "portable-net45+xamarinmac10+xamarinios10")]
         [InlineData(".NETPortable, Version=4.0, Profile=Profile2", "portable-net40+win+xamarinpsthree10+xamarinpsfour10+xamarinpsvita10")]
         [InlineData(".NETPortable, Version=4.0, Profile=Profile3", "portable-net40+xamarinxboxthreesixty10+xamarinxboxone10")]
-        [InlineData(".NETPortable, Version=4.0, Profile=Profile4", "portable-net40+xamarinios10+xamarinwatchos10")]  
+        [InlineData(".NETPortable, Version=4.0, Profile=Profile4", "portable-net40+xamarinios10+xamarinwatchos10")]
         [InlineData(".NETPortable, Version=4.0, Profile=Profile5", "portable-net40+xamarinios10+xamarintvos10")]
         public void TestGetShortNameForPortableXamarinFrameworks(string frameworkIdentifier, string expectedShortName)
         {
@@ -1484,37 +1516,37 @@ namespace NuGet.Test
             var profileCollection = new NetPortableProfileCollection();
             var profile1 = new NetPortableProfile(
                "Profile1",
-               new[] { 
-                           new FrameworkName(".NETFramework, Version=4.5"), 
-                           new FrameworkName("Xamarin.Mac, Version=1.0"), 
-                           new FrameworkName("Xamarin.iOS, Version=1.0"), 
+               new[] {
+                           new FrameworkName(".NETFramework, Version=4.5"),
+                           new FrameworkName("Xamarin.Mac, Version=1.0"),
+                           new FrameworkName("Xamarin.iOS, Version=1.0"),
                       });
 
             var profile2 = new NetPortableProfile(
                "Profile2",
-               new[] { 
-                           new FrameworkName(".NETFramework, Version=4.0"), 
-                           new FrameworkName(".NetCore, Version=4.5"), 
-                           new FrameworkName("Xamarin.PlayStation3, Version=1.0"), 
-                           new FrameworkName("Xamarin.PlayStation4, Version=1.0"), 
-                           new FrameworkName("Xamarin.PlayStationVita, Version=1.0"), 
+               new[] {
+                           new FrameworkName(".NETFramework, Version=4.0"),
+                           new FrameworkName(".NetCore, Version=4.5"),
+                           new FrameworkName("Xamarin.PlayStation3, Version=1.0"),
+                           new FrameworkName("Xamarin.PlayStation4, Version=1.0"),
+                           new FrameworkName("Xamarin.PlayStationVita, Version=1.0"),
                       });
 
             var profile3 = new NetPortableProfile(
                "Profile3",
-               new[] { 
-                           new FrameworkName(".NETFramework, Version=4.0"), 
-                           new FrameworkName("Xamarin.Xbox360, Version=1.0"), 
-                           new FrameworkName("Xamarin.XboxOne, Version=1.0"), 
+               new[] {
+                           new FrameworkName(".NETFramework, Version=4.0"),
+                           new FrameworkName("Xamarin.Xbox360, Version=1.0"),
+                           new FrameworkName("Xamarin.XboxOne, Version=1.0"),
                       });
 
             var profile4 = new NetPortableProfile(
-       "Profile4",
-       new[] {
+                "Profile4",
+                new[] {
                            new FrameworkName(".NETFramework, Version=4.0"),
                            new FrameworkName("Xamarin.iOS, Version=1.0"),
                            new FrameworkName("Xamarin.WatchOS, Version=1.0"),
-              });
+                });
 
             var profile5 = new NetPortableProfile(
                 "Profile5",
@@ -1529,13 +1561,12 @@ namespace NuGet.Test
             profileCollection.Add(profile3);
             profileCollection.Add(profile4);
             profileCollection.Add(profile5);
-
-            NetPortableProfileTable.Profiles = profileCollection;
-
+            
+            var table = new NetPortableProfileTable(profileCollection);
             var framework = new FrameworkName(frameworkIdentifier);
 
             // Act
-            string shortName = VersionUtility.GetShortFrameworkName(framework);
+            string shortName = VersionUtility.GetShortFrameworkName(table, framework);
 
             // Assert
             Assert.Equal(expectedShortName, shortName);
@@ -1619,14 +1650,15 @@ namespace NuGet.Test
         [InlineData("dnx451", "dnxcore50", false)]
         [InlineData("dnxcore50", "dnx451", false)]
 
-        // COMPATIBLE: dnx project, net package (any version)
-        // Don't get excited by version numbers here. I'm just randomly guessing higher version numbers :)
+        // COMPATIBLE: dnx project, earlier or equal net package
         [InlineData("dnx451", "net451", true)]
         [InlineData("dnx451", "net40", true)]
         [InlineData("dnx451", "net20", true)]
-        [InlineData("dnx451", "net50", true)]
-        [InlineData("dnx451", "net60", true)]
-        [InlineData("dnx451", "net70", true)]
+
+        // NOT COMPATIBLE: dnx project, later net package 
+        [InlineData("dnx451", "net50", false)]
+        [InlineData("dnx451", "net60", false)]
+        [InlineData("dnx451", "net70", false)]
 
         // NOT COMPATIBLE: Package targeting later framework
         [InlineData("dnx451", "dnx51", false)]
@@ -1654,28 +1686,30 @@ namespace NuGet.Test
         // COMPATIBLE: Same framework, easy first case
         [InlineData("aspnet50", "aspnet50", true)]
         [InlineData("aspnetcore50", "aspnetcore50", true)]
-        
-        // COMPATIBLE: Project targeting later framework
+
+        // COMPATIBLE: Project targeting earlier or same version
         [InlineData("aspnet51", "aspnet50", true)]
-        [InlineData("aspnet51", "net451", true)]
         [InlineData("aspnet51", "net40", true)]
         [InlineData("aspnet51", "net20", true)]
         [InlineData("aspnetcore51", "aspnetcore50", true)]
-        
+
+        // NOT COMPATIBLE: Project targeting great version
+        [InlineData("aspnet51", "net46", false)]
+        [InlineData("aspnet51", "net451", false)]
+
         // NOT COMPATIBLE: aspnet into aspnetcore and vice-versa
         [InlineData("aspnet50", "aspnetcore50", false)]
         [InlineData("aspnetcore50", "aspnet50", false)]
 
         // COMPATIBLE: aspnet project, net package (any version)
-        // Don't get excited by version numbers here. I'm just randomly guessing higher version numbers :)
-        [InlineData("aspnet50", "net451", true)]
         [InlineData("aspnet50", "net40", true)]
         [InlineData("aspnet50", "net20", true)]
-        [InlineData("aspnet50", "net50", true)]
-        [InlineData("aspnet50", "net60", true)]
-        [InlineData("aspnet50", "net70", true)]
-        
+
         // NOT COMPATIBLE: Package targeting later framework
+        [InlineData("aspnet50", "net451", false)]
+        [InlineData("aspnet50", "net50", false)]
+        [InlineData("aspnet50", "net60", false)]
+        [InlineData("aspnet50", "net70", false)]
         [InlineData("aspnet50", "aspnet51", false)]
         [InlineData("aspnetcore50", "aspnetcore51", false)]
 
@@ -1699,32 +1733,32 @@ namespace NuGet.Test
         public void IsCompatibleMatrixForASPNetFrameworks(string projectFramework, string packageFramework, bool compatible)
         {
             Assert.Equal(
+                compatible,
                 VersionUtility.IsCompatible(
                     VersionUtility.ParseFrameworkName(projectFramework),
-                    VersionUtility.ParseFrameworkName(packageFramework)),
-                compatible);
+                    VersionUtility.ParseFrameworkName(packageFramework)));
         }
 
         [Theory]
         [InlineData("dnx451", "aspnet50", true)]
         [InlineData("dnxcore50", "aspnetcore50", true)]
         [InlineData("aspnet50", "dnx451", false)]
-        [InlineData("aspnetcore50", "dnxcore50", false)]
+        [InlineData("aspnetcore50", "dnxcore50", true)]
         [InlineData("dnx", "aspnet50", true)]
         [InlineData("dnxcore", "aspnetcore50", true)]
         [InlineData("aspnet", "dnx451", false)]
-        [InlineData("aspnetcore", "dnxcore50", false)]
+        [InlineData("aspnetcore", "dnxcore50", true)]
         [InlineData("dnx451", "aspnet", true)]
         [InlineData("dnxcore50", "aspnetcore", true)]
-        [InlineData("aspnet50", "dnx", false)]
-        [InlineData("aspnetcore50", "dnxcore", false)]
+        [InlineData("aspnet50", "dnx", true)]
+        [InlineData("aspnetcore50", "dnxcore", true)]
         public void IsCompatibleMatrixForDNXAspTempFrameworks(string projectFramework, string packageFramework, bool compatible)
         {
             Assert.Equal(
+                compatible,
                 VersionUtility.IsCompatible(
                     VersionUtility.ParseFrameworkName(projectFramework),
-                    VersionUtility.ParseFrameworkName(packageFramework)),
-                compatible);
+                    VersionUtility.ParseFrameworkName(packageFramework)));
         }
 
         [Theory]
@@ -1768,6 +1802,8 @@ namespace NuGet.Test
         [InlineData("portable-wp8+wpa81", "portable-wpa81+wp81")]
         [InlineData("portable-wp81+wpa81", "portable-wpa81+wp81")]
         [InlineData("portable-wpa81+wp81", "portable-wpa81+wp81")]
+        [InlineData("portable-netcore45+sl4+wp", "portable-netcore4+sl4")]
+        [InlineData("portable-netcore45+sl4+wp+net", "portable-wp7+netcore4")]
         public void IsCompatibleReturnsTrueForPortableFrameworkAndPortableFramework(string packageFramework, string projectFramework)
         {
             // Arrange
@@ -1782,9 +1818,7 @@ namespace NuGet.Test
         }
 
         [Theory]
-        [InlineData("portable-netcore45+sl4+wp", "portable-netcore4+sl4")]
         [InlineData("portable-netcore45+sl4+wp", "portable-netcore5+wp7+net")]
-        [InlineData("portable-netcore45+sl4+wp+net", "portable-wp7+netcore4")]
         [InlineData("portable-netcore45+sl4", "portable-net4+wp7")]
         [InlineData("portable-net40+win8+sl4+wp71", "portable-wpa81+wp81")]
         public void IsCompatibleReturnsFalseForPortableFrameworkAndPortableFramework(string packageFramework, string projectFramework)
@@ -1800,198 +1834,37 @@ namespace NuGet.Test
             Assert.False(isCompatible);
         }
 
-        [Theory]
-        [InlineData("portable-net45+sl5+wp71", "portable-net45+sl5+wp71", -3)]
-        [InlineData("portable-net45+sl5+wp71", "portable-net45+sl5+wp71+win8", -4)]
-        [InlineData("portable-net45+sl5+wp71", "portable-net45+sl4+wp71+win8", -54)]
-        [InlineData("portable-net45+sl5+wp71", "portable-net4+sl4+wp71+win8", -104)]
-        [InlineData("portable-net45+sl5+wp71", "portable-net4+sl4+wp7+win8", -154)]
-        [InlineData("portable-win8+wp8", "portable-win8+wp7", -52)]
-        [InlineData("portable-win8+wp8", "portable-win8+wp7+silverlight4", -53)]
-        public void TestGetCompatibilityBetweenPortableLibraryAndPortableLibrary(string frameworkName, string targetFrameworkName, int expectedScore)
-        {
-            // Arrange
-            var framework = VersionUtility.ParseFrameworkName(frameworkName);
-            var targetFramework = VersionUtility.ParseFrameworkName(targetFrameworkName);
-
-            // Act
-            int score = VersionUtility.GetCompatibilityBetweenPortableLibraryAndPortableLibrary(framework, targetFramework);
-
-            // Assert
-            Assert.Equal(expectedScore, score);
-        }
-
-        /// <summary>
-        /// The following example is used in the comments provided in the product code too including how the computation takes place
-        /// Refer VersionUtility.GetCompatibilityBetweenPortableLibraryAndPortableLibrary for more details
-        /// For example, Let Project target net45+sl5+monotouch+monoandroid. And, Package has 4 profiles
-        /// A: net45+sl5, B: net40+sl5+monotouch, C: net40+sl4+monotouch+monoandroid, D: net40+sl4+monotouch+monoandroid+wp71
-        /// </summary>
-        [Theory]
-        [InlineData("portable-net45+sl50+MonoTouch+MonoAndroid", "portable-net45+sl5", -502)]
-        [InlineData("portable-net45+sl50+MonoTouch+MonoAndroid", "portable-net40+sl5+MonoTouch", -303)]
-        [InlineData("portable-net45+sl50+MonoTouch+MonoAndroid", "portable-net40+sl4+MonoTouch+MonoAndroid", -104)]
-        [InlineData("portable-net45+sl50+MonoTouch+MonoAndroid", "portable-net40+sl4+MonoTouch+MonoAndroid+wp71", -105)]
-        public void TestGetCompatibilityBetweenPortableLibraryAndPortableLibraryWithOptionalFx(string frameworkName, string targetFrameworkName, int expectedScore)
-        {            
-            var profile1 = new NetPortableProfile(
-               "Profile1",
-               new[] { 
-                           new FrameworkName(".NETFramework, Version=4.5"), 
-                           new FrameworkName("Silverlight, Version=5.0"), 
-                      },
-               new[] { 
-                           new FrameworkName("MonoTouch, Version=0.0"), 
-                           new FrameworkName("MonoAndroid, Version=0.0"), 
-                      });
-
-            NetPortableProfileCollection profileCollection = new NetPortableProfileCollection();
-            profileCollection.Add(profile1);
-
-            NetPortableProfileTable.Profiles = profileCollection;
-
-            // Arrange
-            var framework = VersionUtility.ParseFrameworkName(frameworkName);
-            var targetFramework = VersionUtility.ParseFrameworkName(targetFrameworkName);
-
-            // Act
-            int score = VersionUtility.GetCompatibilityBetweenPortableLibraryAndPortableLibrary(framework, targetFramework);
-
-            // Assert
-            Assert.Equal(expectedScore, score);
-        }
-
-        /// <summary>
-        /// This test is used to ensure that when the packageTargetFrameworkProfile is already available in NetPortableProfileCollection
-        /// Still the 
-        /// </summary>
-        [Theory]
-        [InlineData("portable-net40+sl40+MonoTouch+MonoAndroid", "portable-net40+sl40+MonoTouch+MonoAndroid", -4)]
-        [InlineData("portable-net45+MonoTouch+MonoAndroid", "portable-net40+sl40+MonoTouch+MonoAndroid", -54)]
-        public void TestGetCompatibilityBetweenPortableLibraryAndPortableLibraryWithPreLoadedPackageProfile(string frameworkName, string targetFrameworkName, int expectedScore)
-        {
-            var profile1 = new NetPortableProfile(
-               "Profile1",
-               new[] { 
-                           new FrameworkName(".NETFramework, Version=4.0"), 
-                           new FrameworkName("Silverlight, Version=4.0"), 
-                      },
-               new[] { 
-                           new FrameworkName("MonoTouch, Version=0.0"), 
-                           new FrameworkName("MonoAndroid, Version=0.0"), 
-                      });
-
-            NetPortableProfileCollection profileCollection = new NetPortableProfileCollection();
-            profileCollection.Add(profile1);
-
-            NetPortableProfileTable.Profiles = profileCollection;
-
-            // Arrange
-            var framework = VersionUtility.ParseFrameworkName(frameworkName);
-            var targetFramework = VersionUtility.ParseFrameworkName(targetFrameworkName);
-
-            // Act
-            int score = VersionUtility.GetCompatibilityBetweenPortableLibraryAndPortableLibrary(framework, targetFramework);
-
-            // Assert
-            Assert.Equal(expectedScore, score);
-        }
-
-        /// <summary>
-        /// (a)  First case is when projectFrameworkName is not compatible with packageTargetFrameworkName and returns long.MinValue
-        /// (b)  Second case is where there is a framework in portable packageFramework compatible with the Mono projectFramework
-        /// (c)  The last cases are when there is no framework in portable packageFrameowrk that is compatible with the Mono projectFramework
-        ///      (i)   Check if there is an *installed* portable profile which has the aforementioned project framework as an optional framework        
-        ///      (ii)  And, check if the project framework version >= found optional framework and that the supported frameworks are compatible with the ones in packageTargetFramework
-        ///      (iii) In the source code, this is the else part in method GetCompatibilityBetweenPortableLibraryAndNonPortableLibrary()
-        /// </summary>
-        [Theory]
-        [InlineData("MonoAndroid10", "portable-net45+sl5", long.MinValue)]
-        // 180388626433 below = (1L << 32 + 5) + 1 + (10 * (1L << 32)). And, this is the score accumulated 
-        // across methods like CalculateVersionDistance and GetProfileCompatibility
-        [InlineData("MonoAndroid10", "portable-net40+sl4+wp71+win8+MonoAndroid10", (180388626433 - 5 * 2))]
-        [InlineData("MonoAndroid10", "portable-net40+sl4+wp71+win8", -4*2)]
-        [InlineData("MonoAndroid10", "portable-net45+wp8+win8", -3*2)]
-        [InlineData("MonoAndroid10", "portable-net40+sl4+wp71+win8+MonoTouch", -5*2)]
-        [InlineData("MonoAndroid20", "portable-net40+sl4+wp71+win8+MonoTouch", -5 * 2)]
-        [InlineData("MonoAndroid", "portable-net40+sl4+wp71+win8+MonoTouch", long.MinValue)]
-        public void TestGetCompatibilityBetweenPortableLibraryAndNonPortableLibraryForMono(string projectFrameworkName, string packageTargetFrameworkName, long expectedScore)        
-        {
-            // Arrange
-            var profile1 = new NetPortableProfile(
-               "Profile1",
-               new[] { 
-                           new FrameworkName(".NETFramework, Version=4.5"), 
-                           new FrameworkName("Silverlight, Version=4.0"),
-                           new FrameworkName("WindowsPhone, Version=7.1"),
-                           new FrameworkName("Windows, Version=8.0"),
-                      },
-               new[] { 
-                           new FrameworkName("MonoTouch, Version=1.0"), 
-                           new FrameworkName("MonoAndroid, Version=1.0"),
-                           new FrameworkName("MonoMac, Version=1.0"),
-                      });
-
-            var profile2 = new NetPortableProfile(
-               "Profile2",
-               new[] { 
-                           new FrameworkName(".NETFramework, Version=4.5"), 
-                           new FrameworkName("WindowsPhone, Version=8.0"),
-                           new FrameworkName("Windows, Version=8.0"),
-                      },
-               new[] { 
-                           new FrameworkName("MonoTouch, Version=1.0"), 
-                           new FrameworkName("MonoAndroid, Version=1.0"),
-                      });
-
-            NetPortableProfileCollection profileCollection = new NetPortableProfileCollection();
-            profileCollection.Add(profile1);
-            profileCollection.Add(profile2);
-
-            NetPortableProfileTable.Profiles = profileCollection;
-
-            // Arrange
-            var framework = VersionUtility.ParseFrameworkName(projectFrameworkName);
-            var targetFramework = VersionUtility.ParseFrameworkName(packageTargetFrameworkName);
-
-            // Act
-            long score = VersionUtility.GetCompatibilityBetweenPortableLibraryAndNonPortableLibrary(framework, targetFramework);
-
-            // Assert
-            Assert.Equal(expectedScore, score);
-        }
-
         private NetPortableProfileCollection BuildProfileCollection()
         {
             var profileCollection = new NetPortableProfileCollection();
             var profile1 = new NetPortableProfile(
                "Profile1",
-               new[] { 
-                           new FrameworkName(".NETFramework, Version=4.5"), 
-                           new FrameworkName("Silverlight, Version=4.0"), 
-                           new FrameworkName("WindowsPhone, Version=7.1"), 
+               new[] {
+                           new FrameworkName(".NETFramework, Version=4.5"),
+                           new FrameworkName("Silverlight, Version=4.0"),
+                           new FrameworkName("WindowsPhone, Version=7.1"),
                       });
 
             var profile2 = new NetPortableProfile(
                "Profile2",
-               new[] { 
-                           new FrameworkName(".NetCore, Version=4.5"), 
-                           new FrameworkName("Silverlight, Version=3.0"), 
-                           new FrameworkName("WindowsPhone, Version=7.1"), 
+               new[] {
+                           new FrameworkName(".NetCore, Version=4.5"),
+                           new FrameworkName("Silverlight, Version=3.0"),
+                           new FrameworkName("WindowsPhone, Version=7.1"),
                       });
 
             var profile3 = new NetPortableProfile(
                "Profile3",
-               new[] { 
-                           new FrameworkName(".NetCore, Version=4.5"), 
-                           new FrameworkName(".NETFramework, Version=2.0"), 
+               new[] {
+                           new FrameworkName(".NetCore, Version=4.5"),
+                           new FrameworkName(".NETFramework, Version=2.0"),
                       });
 
             var profile4 = new NetPortableProfile(
                "Profile4",
-               new[] { 
-                           new FrameworkName("Silverlight, Version=2.0"), 
-                           new FrameworkName("Silverlight, Version=3.0, Profile=WindowsPhone"), 
+               new[] {
+                           new FrameworkName("Silverlight, Version=2.0"),
+                           new FrameworkName("Silverlight, Version=3.0, Profile=WindowsPhone"),
                       });
 
             profileCollection.Add(profile1);
@@ -2007,6 +1880,12 @@ namespace NuGet.Test
         [InlineData("dotnet10", ".NETPlatform", "1.0")]
         [InlineData("dotnet50", ".NETPlatform", "5.0")]
         [InlineData("dotnet60", ".NETPlatform", "6.0")]
+        [InlineData("netstandard", ".NETStandard", "0.0")]
+        [InlineData("netstandard10", ".NETStandard", "1.0")]
+        [InlineData("netstandardapp", ".NETStandardApp", "0.0")]
+        [InlineData("netstandardapp10", ".NETStandardApp", "1.0")]
+        [InlineData("netcoreapp", ".NETCoreApp", "1.0")]
+        [InlineData("netcoreapp10", ".NETCoreApp", "1.0")]
         public void CanParseShortFrameworkNames(string shortName, string longName, string version)
         {
             var fx = VersionUtility.ParseFrameworkName(shortName);
@@ -2016,6 +1895,12 @@ namespace NuGet.Test
         [Theory]
         [InlineData(".NETPlatform", "0.0", "dotnet")]
         [InlineData(".NETPlatform", "5.0", "dotnet")]
+        [InlineData(".NETStandard", "0.0", "netstandard")]
+        [InlineData(".NETStandard", "1.0", "netstandard1.0")]
+        [InlineData(".NETStandardApp", "0.0", "netstandardapp")]
+        [InlineData(".NETStandardApp", "1.0", "netstandardapp1.0")]
+        [InlineData(".NETCoreApp", "0.0", "netcoreapp")]
+        [InlineData(".NETCoreApp", "1.0", "netcoreapp1.0")]
         public void ShortFrameworkNamesAreCorrect(string longName, string version, string shortName)
         {
             var fx = new FrameworkName(longName, Version.Parse(version));
@@ -2025,6 +1910,9 @@ namespace NuGet.Test
         [Theory]
         [InlineData(".NETPlatform5.0", ".NETPlatform", "5.0")]
         [InlineData(".NETPlatform50", ".NETPlatform", "5.0")]
+        [InlineData(".NETStandard10", ".NETStandard", "1.0")]
+        [InlineData(".NETStandardApp10", ".NETStandardApp", "1.0")]
+        [InlineData(".NETCoreApp10", ".NETCoreApp", "1.0")]
         public void CanParseMixedFrameworkNames(string mixedName, string longName, string version)
         {
             var fx = VersionUtility.ParseFrameworkName(mixedName);
@@ -2034,6 +1922,11 @@ namespace NuGet.Test
         [Theory]
         [InlineData(".NETPlatform5.0", "dotnet")]
         [InlineData(".NETPlatform50", "dotnet")]
+        [InlineData(".NETStandard", "netstandard")]
+        [InlineData(".NETStandardApp", "netstandardapp")]
+        [InlineData(".NETStandard1.0", "netstandard1.0")]
+        [InlineData(".NETStandardApp1.0", "netstandardapp1.0")]
+        [InlineData(".NETCoreApp1.0", "netcoreapp1.0")]
         public void CanParseMixedFrameworkNamesToShort(string mixedName, string shortName)
         {
             var fx = VersionUtility.ParseFrameworkName(mixedName);
@@ -2041,5 +1934,47 @@ namespace NuGet.Test
 
             Assert.Equal(shortName, result);
         }
+
+        /// <summary>
+        /// Based on GitHub issue: https://github.com/NuGet/Home/issues/3410
+        /// </summary>
+        [Fact]
+        public void CanReduceDuplicateFrameworks()
+        {
+            // Arrange
+            var projectFramework = VersionUtility.ParseFrameworkName("net452");
+            var expectedFramework = VersionUtility.ParseFrameworkName("net45");
+            var frameworks = new[]
+            {
+                "net40",
+                "net45",
+                "netstandard1.0",
+                "portable45-net45+win8+wp8+wpa81", // This framework and the next are duplicate.
+                "portable-net45+win+wpa81+wp80+MonoAndroid10+MonoTouch10"
+            };
+
+            var items = frameworks
+                .Select(fn =>
+                {
+                    var mock = new Mock<IFrameworkTargetable>();
+
+                    mock
+                        .Setup(x => x.SupportedFrameworks)
+                        .Returns(() => new[] { VersionUtility.ParseFrameworkName(fn) });
+
+                    return mock.Object;
+                });
+
+            IEnumerable<IFrameworkTargetable> compatible;
+
+            // Act
+            var result = VersionUtility.TryGetCompatibleItems(projectFramework, items, out compatible);
+
+            // Assert
+            Assert.True(result, "Compatible items should have been found.");
+            Assert.Equal(1, compatible.Count());
+            Assert.Equal(1, compatible.First().SupportedFrameworks.Count());
+            Assert.Equal(expectedFramework, compatible.First().SupportedFrameworks.First());
+        }
     }
 }
